﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace DL.PhotoCollage.Core
{
    public class RandomFileSystemPhotoRepository : IPhotoRepository
    {
        private readonly ConcurrentQueue<string> photoFilePaths;

        private readonly List<string> displayedPhotos;

        private readonly string rootDirectoryPath;

        private readonly object threadLock = new object();

        public RandomFileSystemPhotoRepository(string path)
        {
            this.photoFilePaths = new ConcurrentQueue<string>();
            this.displayedPhotos = new List<string>();
            this.rootDirectoryPath = path;

            this.LoadPathsFromFileSystem();
        }

        public bool HasPhotos
        {
            get { return !this.photoFilePaths.IsEmpty; }
        }

        public string NextPhotoFilePath
        {
            get
            {
                string path;

                if (!this.photoFilePaths.TryDequeue(out path))
                {
                    this.ReloadPhotoQueue();
                    this.photoFilePaths.TryDequeue(out path);
                }

                lock (this.threadLock)
                {
                    this.displayedPhotos.Add(path);
                }

                return Path.Combine(this.rootDirectoryPath, path);
            }
        }

        private static IEnumerable<string> RandomizePaths(IEnumerable<string> paths)
        {
            var random = new Random();
            return paths.OrderBy(item => random.Next());
        }

        private static IEnumerable<string> GetPhotoFileExtensions()
        {
            return new List<string> { "*.jpg", "*.jpeg", "*.png" };
        }

        private void ReloadPhotoQueue()
        {
            lock (this.threadLock)
            {
                IEnumerable<string> photosToQueue = RandomizePaths(this.displayedPhotos);
                this.LoadPhotoPathsIntoQueue(photosToQueue);
                this.displayedPhotos.Clear();
            }
        }

        private void LoadPathsFromFileSystem()
        {
            var paths = new List<string>();
            var directory = new DirectoryInfo(this.rootDirectoryPath);
            var extensions = GetPhotoFileExtensions();

            foreach (string extension in extensions)
            {
                var pathsFromExtension
                    = from file in directory.GetFiles(extension, SearchOption.AllDirectories).AsParallel()
                      select file.FullName.Replace(this.rootDirectoryPath, string.Empty).TrimStart(new[] { '\\' });
                paths.AddRange(pathsFromExtension);
            }

            var randomizedPaths = RandomizePaths(paths);
            this.LoadPhotoPathsIntoQueue(randomizedPaths);
        }

        private void LoadPhotoPathsIntoQueue(IEnumerable<string> paths)
        {
            foreach (var path in paths)
            {
                this.photoFilePaths.Enqueue(path);
            }
        }
    }
}
