﻿using System;
using System.Collections.Concurrent;
using System.Windows;
using System.Windows.Threading;
using DL.PhotoCollage.Core;
using DL.PhotoCollage.Wpf.UserControls;

namespace DL.PhotoCollage.Wpf.Presentation
{
    public abstract class CollagePresenterBase
    {
        protected CollagePresenterBase(ControllerBase controller)
        {
            this.Controller = controller;
        }

        protected ControllerBase Controller { get; set; }

        protected IPhotoRepository PhotoRespository { get; set; }

        protected int MaxInQueue { get; set; }

        protected ConcurrentQueue<ImageDisplayUserControl> ImageQueue { get; set; }

        public void StartAnimation()
        {
            try
            {
                if (!this.PhotoRespository.HasPhotos)
                {
                    this.Controller.DisplayErrorMessage("Folder does not contain any supported photos.");
                    this.Controller.ShutdownApplication();
                }

                this.DisplayImageTimerTick(null, null);

                int seconds = (int)this.Controller.Configuration.Speed;
                var timer = new DispatcherTimer { Interval = new TimeSpan(0, 0, seconds) };
                timer.Tick += this.DisplayImageTimerTick;
                timer.Start();
            }
            catch (Exception ex)
            {
                this.Controller.HandleError(ex);
            }
        }

        protected abstract void AddControlToView(ImageDisplayUserControl control);

        protected abstract bool DoesViewContainControl(ImageDisplayUserControl control);

        protected abstract double GetViewActualHeight();

        protected abstract double GetViewActualWidth();

        protected abstract void RemoveControlFromView(ImageDisplayUserControl control);

        private void DisplayImageTimerTick(object sender, EventArgs e)
        {
            try
            {
                string path = this.PhotoRespository.NextPhotoFilePath;

                var control = new ImageDisplayUserControl(this.Controller, path);
                this.AddControlToView(control);
                this.ImageQueue.Enqueue(control);

                if (this.ImageQueue.Count > this.MaxInQueue)
                {
                    this.RemoveImageFromQueue();
                }

                this.SetUserControlPosition(control);
            }
            catch (Exception ex)
            {
                this.Controller.HandleError(ex);
                this.Controller.ShutdownApplication();
            }
        }

        private void SetUserControlPosition(UIElement control)
        {
            double viewportWidth = this.GetViewActualWidth();
            double viewportHeight = this.GetViewActualHeight();

            var positioner = new ImagePositioner(this.Controller, control, viewportWidth, viewportHeight);
            positioner.Position();
        }

        private void RemoveImageFromQueue()
        {
            ImageDisplayUserControl control;

            if (this.ImageQueue.TryDequeue(out control))
            {
                Action<ImageDisplayUserControl> action = this.RemoveImageFromPanel;
                control.FadeOutImage(action);
            }
        }

        private void RemoveImageFromPanel(ImageDisplayUserControl control)
        {
            try
            {
                if (this.DoesViewContainControl(control))
                {
                    this.RemoveControlFromView(control);
                }
            }
            catch (Exception ex)
            {
                this.Controller.HandleError(ex);
            }
        }
    }
}
