﻿using System;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using DL.PhotoCollage.Core;
using DL.PhotoCollage.Wpf.Presentation;

namespace DL.PhotoCollage.Wpf.UserControls
{
    public partial class ImageDisplayUserControl : UserControl
    {
        private readonly ImageDisplayPresenter presenter;

        private readonly string filePath;
        
        public ImageDisplayUserControl(ControllerBase applicationController, string path)
        {
            try
            {
                this.presenter = new ImageDisplayPresenter(applicationController);
                this.filePath = path;
                this.InitializeComponent();
            }
            catch (Exception ex)
            {
                applicationController.HandleError(ex);
            }            
        }

        public void FadeOutImage(Action<ImageDisplayUserControl> onCompletedAction)
        {
            try
            {
                var storyboard = new Storyboard();
                var duration = new TimeSpan(0, 0, 1);

                var animation
                    = new DoubleAnimation
                    {
                        From = 1.0,
                        To = 0.0,
                        Duration = new Duration(duration)
                    };

                storyboard.Completed 
                    += delegate
                        {
                            onCompletedAction(this);
                        };

                Storyboard.SetTargetName(animation, this.MainBorder.Name);
                Storyboard.SetTargetProperty(animation, new PropertyPath(OpacityProperty));
                storyboard.Children.Add(animation);
                storyboard.Begin(this);
            }
            catch (Exception ex)
            {
                this.presenter.HandleError(ex);
            }
        }

        private void ImageDisplayUserControl_OnInitialized(object sender, EventArgs e)
        {
            try
            {
                if (!this.presenter.ShowBorders)
                {
                    this.MainBorder.BorderThickness = new Thickness(0);
                    this.InnerBorder.BorderThickness = new Thickness(0);
                }

                this.MainImage.MaxHeight = this.presenter.MaximumSizePixels;
                this.MainImage.MaxWidth = this.presenter.MaximumSizePixels;

                this.RotateImage();
                this.LoadImage();
            }
            catch (Exception ex)
            {
                this.presenter.HandleError(ex);
            }
        }

        private void RotateImage()
        {
            int angle = this.presenter.GetRandomAngle();
            var transform = new RotateTransform(angle);
            this.MainBorder.RenderTransform = transform;
        }

        private void LoadImage()
        {
            BitmapSource rawImage = this.GetRawImage();

            if (this.presenter.IsGrayscale)
            {
                FormatConvertedBitmap grayImage = this.GetGrayscaleImage(rawImage);
                this.presenter.CalculateDpiScale(grayImage.DpiX);
                this.MainImage.Source = this.GetDisplayImage(grayImage);    
            }
            else
            {
                this.presenter.CalculateDpiScale(rawImage.DpiX);
                this.MainImage.Source = this.GetDisplayImage(rawImage);    
            }           
        }

        private BitmapImage GetRawImage()
        {
            var image = new BitmapImage();
            image.BeginInit();
            image.CacheOption = BitmapCacheOption.None;
            image.CreateOptions = BitmapCreateOptions.IgnoreColorProfile | BitmapCreateOptions.IgnoreImageCache;
            image.UriSource = new Uri(this.filePath, UriKind.Absolute);
            image.EndInit();
            return image;
        }

        private FormatConvertedBitmap GetGrayscaleImage(BitmapSource source)
        {
            var image = new FormatConvertedBitmap();
            image.BeginInit();
            image.Source = source;
            image.DestinationFormat = PixelFormats.Gray32Float;
            image.EndInit();
            return image;
        }

        private ImageSource GetDisplayImage(BitmapSource rawImage)
        {
            if (this.presenter.DoesImageNeedScaling(rawImage.Height, rawImage.Width))
            {
                double scale = this.GetScale(rawImage);
                return this.GetScaledImage(rawImage, scale);
            }
            
            return rawImage;
        }

        private TransformedBitmap GetScaledImage(BitmapSource original, double scale)
        {
            var transform = new ScaleTransform(scale, scale);
            return new TransformedBitmap(original, transform);
        }

        private double GetScale(ImageSource image)
        {
            ImageOrientation orientation = this.presenter.GetImageOrientation(image.Height, image.Width);
            return orientation == ImageOrientation.Portrait
                    ? this.presenter.GetPortraitScale(image.Height)
                    : this.presenter.GetLandscapeScale(image.Width);
        }
    }
}
